USE [igops]
GO

DROP VIEW IF EXISTS review_instance_v;
DROP VIEW IF EXISTS review_definition_v;
DROP VIEW IF EXISTS review_definition_all_v;
DROP VIEW IF EXISTS reviewer_definition_v;
DROP VIEW IF EXISTS review_notifications_v;
DROP VIEW IF EXISTS review_definition_details_v;
GO

CREATE VIEW review_instance_v AS
  SELECT
    review_instance.id,
    review_instance.certification_date,
    review_instance.end_date,
    review_instance.escalation_timout_milliseconds,
    review_instance.expected_end_date,
    review_instance.last_status_change,
    review_instance.process_id,
    review_instance.start_date,
    review_instance.status,
    review_instance.status_comment,
    review_instance.valid_to_date,
    review_instance.wf_sync_failed,
    review_instance.review_definition,
    review_instance.task_count,
    review_instance.task_complete_count,
    review_instance.item_count,
    review_instance.review_type,
    review_instance.canceled_by,
    defn.active_from_date,
    defn.unique_review_def_id
  FROM review_instance
    JOIN (SELECT
        review_instance.id inst_id,
        defn.id defn_id, 
        defn.unique_review_def_id,
        CASE WHEN staged_defn.active_from_date IS NULL THEN defn.active_from_date ELSE staged_defn.active_from_date END AS active_from_date
      FROM review_definition defn
        JOIN review_instance ON defn.id = review_instance.review_definition
        LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn ON defn.inst_id = review_instance.id
  WHERE review_instance.status NOT IN ('PREVIEW', 'PREVIEW_CANCELED', 'PREVIEW_STARTING')
GO

CREATE VIEW reviewer_definition_v AS
  SELECT DISTINCT 
    reviewer_definition.review_definition_id definition_id,
    reviewer_definition.review_order,
    reviewer_definition.type,
    review_users.rev_users review_user,
    sgroup.name review_group,
    coverage_map.name coverage_name
  FROM reviewer_definition
    LEFT JOIN reviewer_set_definition ON reviewer_set_definition.reviewer_definition_id = reviewer_definition.id
    LEFT JOIN sgroup ON sgroup.unique_group_id = reviewer_set_definition.unique_group_id
    LEFT JOIN coverage_map ON coverage_map.coverage_map_id = reviewer_set_definition.coverage_map_id
    LEFT JOIN (
    SELECT DISTINCT 
      defn.name review_name, 
      defn.id,
      rdefn.type,
    STUFF((SELECT ' | ' + s1.display_name
            FROM review_definition defn1
        JOIN reviewer_definition rdefn1 ON defn1.id = rdefn1.review_definition_id
        LEFT JOIN reviewer_set_definition rsd1 ON rdefn1.id = rsd1.reviewer_definition_id
        LEFT JOIN suser s1 ON rsd1.unique_user_id = s1.unique_user_id AND s1.effective = 1
        WHERE defn1.id = defn.id AND rdefn1.type = rdefn.type
            FOR XML PATH('')),1,3,'') rev_users
    FROM review_definition defn
      JOIN reviewer_definition rdefn ON defn.id = rdefn.review_definition_id
      LEFT JOIN reviewer_set_definition rsd ON rdefn.id = rsd.reviewer_definition_id
      LEFT JOIN suser s ON rsd.unique_user_id = s.unique_user_id AND s.effective = 1
    GROUP BY defn.name, defn.id, rdefn.type
    ) review_users ON review_users.id = reviewer_definition.review_definition_id AND review_users.type = reviewer_definition.type
GO


CREATE VIEW review_definition_v AS 
  SELECT
    review_definition.unique_review_def_id,
    review_definition.active_from_date,
    review_definition.active_to_date,
    review_definition.name,
    review_definition.review_type,
    review_definition.id,
    review_definition.description,
    review_definition.instructions,
    review_definition.allow_override,
    review_definition.include_accounts,
    review_definition.show_role_permissions,
    review_definition.review_target_spec_json,
    review_definition.duration,
    review_definition.duration_unit,
    review_definition.valid_for,
    review_definition.valid_for_unit,
    review_definition.escalation_timeout,
    review_definition.escalation_timeout_unit,
    review_definition.next_schedule_date,
    review_definition.end_reminder,
    review_definition.end_reminder_unit,
    review_definition.repeat,
    review_definition.repeat_unit,
    review_definition.repeat_after,
    review_definition.time_zone_id,
    review_definition.schedule_active,
    review_definition.deleted,
    review_definition.partial_approval_policy,
    review_definition.expiration_policy,
    review_definition.expiration_extension_unit,
    review_definition.expiration_extension,
    review_definition.allow_modify_access,
    review_definition.allow_self_review,
    review_definition.exclude_authorized,
    review_definition.hide_reassign,
    review_definition.include_permissions,
    review_definition.show_bus_role_policies,
    review_definition.show_acct_permissions,
    review_owners.display_name review_owners,
    esc_reviewer.esc_reviewers,
    auditor.review_user auditor
  FROM review_definition
    LEFT JOIN reviewer_definition_v auditor ON review_definition.id = auditor.definition_id AND auditor.type = 'AUDITOR'
    LEFT JOIN (SELECT 
        rdefn1.definition_id,
        STUFF((SELECT ' | ' + review_user
          FROM reviewer_definition_v  rdefn2
          WHERE rdefn2.type = 'EO' and rdefn2.definition_id = rdefn1.definition_id
      ORDER BY review_user
          FOR XML PATH('')), 1, 3, '') AS esc_reviewers
      FROM reviewer_definition_v rdefn1 
      WHERE rdefn1.type = 'EO'
      GROUP BY rdefn1.definition_id) esc_reviewer ON esc_reviewer.definition_id = review_definition.id
    LEFT JOIN ( SELECT 
        defn_1.unique_review_def_id id,
        STUFF((SELECT ' | ' + s.display_name
          FROM  review_definition rdf
            JOIN reviewer_definition on rdf.id = reviewer_definition.review_definition_id and reviewer_definition.type = 'RO'
            JOIN reviewer_set_definition on reviewer_definition.id = reviewer_set_definition.reviewer_definition_id
            JOIN suser s ON reviewer_set_definition.unique_user_id = s.unique_user_id AND s.effective = 1
          WHERE rdf.is_active = '1' AND rdf.unique_review_def_id = defn_1.unique_review_def_id
      ORDER BY s.display_name
          FOR XML PATH('')),1,3,'') display_name
      FROM  review_definition defn_1 
        JOIN reviewer_definition on defn_1.id = reviewer_definition.review_definition_id and reviewer_definition.type = 'RO'
        JOIN reviewer_set_definition on reviewer_definition.id = reviewer_set_definition.reviewer_definition_id
        JOIN suser s ON reviewer_set_definition.unique_user_id = s.unique_user_id AND s.effective = 1
        WHERE defn_1.is_active = '1'
        GROUP BY defn_1.unique_review_def_id) review_owners ON review_owners.id = review_definition.unique_review_def_id
  WHERE review_definition.is_active = 1
GO


CREATE VIEW review_notifications_v AS
  SELECT DISTINCT
    def.name review_name,
    rev_notify.review_def_id review_id,
    rev_notify.id notification_id,
    notify.name notification_name,
    notify.description notification_description
  FROM review_notify_defn rev_notify
    LEFT JOIN notify_definition notify ON notify.id = rev_notify.id
    LEFT JOIN review_definition def ON def.id = rev_notify.review_def_id
GO


CREATE VIEW review_definition_details_v AS 
  SELECT
    defn.name,
    defn.unique_review_def_id,
    defn.deleted,
    canceled_by.display_name canceled_by,
    rev_inst.task_complete_count,
    rev_inst.task_count,
    rev_inst.start_date,
    rev_inst.end_date,
    rev_inst.expected_end_date,
    rev_inst.status,
    rev_inst.status_comment,
    multi_owner.review_owners
  FROM (SELECT
      review_instance.id inst_id,
      defn.id defn_id, 
      defn.unique_review_def_id,
      CASE WHEN staged_defn.name IS NULL THEN defn.name ELSE staged_defn.name END AS name,
      defn.deleted
    FROM review_definition defn
      JOIN review_instance ON defn.id = review_instance.review_definition
      LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn 
    JOIN review_instance rev_inst ON defn.inst_id = rev_inst.id AND rev_inst.status NOT IN ('PREVIEW', 'PREVIEW_CANCELED', 'PREVIEW_STARTING')
    LEFT JOIN suser canceled_by ON rev_inst.canceled_by = canceled_by.unique_user_id AND canceled_by.effective = 1
    JOIN ( SELECT DISTINCT defn_1.name review_name,
        ri.id rid,
        STUFF((SELECT DISTINCT ' | ' + s.display_name FROM review_instance ri1
          JOIN review_definition defn_1 ON ri1.review_definition = defn_1.id
          LEFT JOIN reviewer_instance rev_owner ON ri1.id = rev_owner.review_instance_id AND rev_owner.type = 'RO'
          LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = rev_owner.id
          LEFT JOIN suser s ON rui.unique_user_id = s.unique_user_id AND s.effective = 1 
          WHERE ri1.ID = ri.id
          FOR XML PATH('')),1,3,'') review_owners
      FROM review_instance ri
        JOIN review_definition defn_1 ON ri.review_definition = defn_1.id
        LEFT JOIN reviewer_instance rev_owner ON ri.id = rev_owner.review_instance_id AND rev_owner.type = 'RO'
        LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = rev_owner.id
        LEFT JOIN suser s ON rui.unique_user_id = s.unique_user_id AND s.effective = 1
      GROUP BY defn_1.name, ri.id) multi_owner ON rev_inst.id = multi_owner.rid
GO


GRANT SELECT ON review_instance_v TO ig_report_role
GRANT SELECT ON review_definition_v TO ig_report_role
GRANT SELECT ON reviewer_definition_v TO ig_report_role
GRANT SELECT ON review_notifications_v TO ig_report_role
GRANT SELECT ON review_definition_details_v TO ig_report_role
GO