DROP VIEW IF EXISTS review_instance_v;
DROP VIEW IF EXISTS review_definition_v;
DROP VIEW IF EXISTS review_definition_all_v;
DROP VIEW IF EXISTS reviewer_definition_v;
DROP VIEW IF EXISTS review_notifications_v;
DROP VIEW IF EXISTS review_definition_details_v;

CREATE VIEW review_instance_v AS
  SELECT
    review_instance.id,
    review_instance.certification_date,
    review_instance.end_date,
    review_instance.escalation_timout_milliseconds,
    review_instance.expected_end_date,
    review_instance.last_status_change,
    review_instance.process_id,
    review_instance.start_date,
    review_instance.status,
    review_instance.status_comment,
    review_instance.valid_to_date,
    review_instance.wf_sync_failed,
    review_instance.review_definition,
    review_instance.task_count,
    review_instance.task_complete_count,
    review_instance.item_count,
    review_instance.review_type,
    review_instance.canceled_by,
    defn.active_from_date,
    defn.unique_review_def_id
  FROM review_instance
    JOIN (SELECT
        review_instance.id inst_id,
        defn.id defn_id, 
        defn.unique_review_def_id,
        CASE WHEN staged_defn.active_from_date IS NULL THEN defn.active_from_date ELSE staged_defn.active_from_date END AS active_from_date
      FROM review_definition defn
        JOIN review_instance ON defn.id = review_instance.review_definition
        LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn ON defn.inst_id = review_instance.id
  WHERE review_instance.status NOT IN ('PREVIEW', 'PREVIEW_CANCELED', 'PREVIEW_STARTING');

CREATE VIEW reviewer_definition_v AS
  SELECT DISTINCT 
    reviewer_definition.review_definition_id definition_id,
    reviewer_definition.review_order,
    reviewer_definition.type,
    review_users.rev_users review_user,
    sgroup.name review_group,
    coverage_map.name coverage_name
  FROM reviewer_definition
    LEFT JOIN reviewer_set_definition ON reviewer_set_definition.reviewer_definition_id = reviewer_definition.id
    LEFT JOIN suser ON suser.unique_user_id = reviewer_set_definition.unique_user_id
    LEFT JOIN sgroup ON sgroup.unique_group_id = reviewer_set_definition.unique_group_id
    LEFT JOIN coverage_map ON coverage_map.coverage_map_id = reviewer_set_definition.coverage_map_id
    LEFT JOIN 
    (SELECT DISTINCT 
      defn.name review_name, 
      defn.id,
      rdefn.type,
      string_agg(s.display_name, ' | ') AS rev_users
    FROM review_definition defn
      JOIN reviewer_definition rdefn ON defn.id = rdefn.review_definition_id
      LEFT JOIN reviewer_set_definition rsd ON rdefn.id = rsd.reviewer_definition_id
      LEFT JOIN suser s ON rsd.unique_user_id = s.unique_user_id AND s.effective = true
    GROUP BY defn.name, defn.id, rdefn.type) review_users ON review_users.id = reviewer_definition.review_definition_id AND review_users.type = reviewer_definition.type;

CREATE VIEW review_definition_v AS 
  SELECT
    review_definition.unique_review_def_id,
    review_definition.active_from_date,
    review_definition.active_to_date,
    review_definition.name,
    review_definition.review_type,
    review_definition.id,
    review_definition.description,
    review_definition.instructions,
    review_definition.allow_override,
    review_definition.include_accounts,
    review_definition.show_role_permissions,
    review_definition.review_target_spec_json,
    review_definition.duration,
    review_definition.duration_unit,
    review_definition.valid_for,
    review_definition.valid_for_unit,
    review_definition.escalation_timeout,
    review_definition.escalation_timeout_unit,
    review_definition.next_schedule_date,
    review_definition.end_reminder,
    review_definition.end_reminder_unit,
    review_definition.repeat,
    review_definition.repeat_unit,
    review_definition.repeat_after,
    review_definition.time_zone_id,
    review_definition.schedule_active,
    review_definition.deleted,
    review_definition.partial_approval_policy,
    review_definition.expiration_policy,
    review_definition.expiration_extension_unit,
    review_definition.expiration_extension,
    review_definition.allow_modify_access,
    review_definition.allow_self_review,
    review_definition.exclude_authorized,
    review_definition.hide_reassign,
    review_definition.include_permissions,
    review_definition.show_bus_role_policies,
    review_definition.show_acct_permissions,
    review_owners.display_name review_owners,
    esc_reviewer.esc_reviewers,
    auditor.review_user auditor
  FROM review_definition
    LEFT JOIN reviewer_definition_v auditor ON review_definition.id = auditor.definition_id AND auditor.type = 'AUDITOR'
    LEFT JOIN (SELECT 
        rdefn1.definition_id,
        string_agg(rdefn1.review_user, '|' ORDER BY rdefn1.review_user) AS esc_reviewers
      FROM reviewer_definition_v rdefn1
      WHERE rdefn1.type = 'EO'
      GROUP BY rdefn1.definition_id) esc_reviewer ON esc_reviewer.definition_id = review_definition.id
    JOIN ( SELECT 
        rdf.unique_review_def_id id,
        string_agg(s.display_name,' | ' ORDER BY s.display_name) as display_name
      FROM review_definition rdf
        JOIN reviewer_definition on rdf.id = reviewer_definition.review_definition_id and reviewer_definition.type = 'RO'
        JOIN reviewer_set_definition ON reviewer_definition.id = reviewer_set_definition.reviewer_definition_id
        JOIN suser s ON reviewer_set_definition.unique_user_id = s.unique_user_id AND s.effective = true
      WHERE rdf.is_active = true
      GROUP BY rdf.unique_review_def_id) review_owners ON review_owners.id = review_definition.unique_review_def_id
  WHERE review_definition.is_active = true;

CREATE VIEW review_notifications_v AS 
 SELECT DISTINCT def.name AS review_name,
    rev_notify.review_def_id AS review_id,
    rev_notify.id AS notification_id,
    notify.name AS notification_name,
    notify.description AS notification_description
   FROM review_notify_defn rev_notify
     LEFT JOIN notify_definition notify ON notify.id = rev_notify.id
     LEFT JOIN review_definition def ON def.id = rev_notify.review_def_id;

CREATE VIEW review_definition_details_v AS
  SELECT 
    defn.name,
    defn.unique_review_def_id,
    defn.deleted,
    canceled_by.display_name AS canceled_by,
    rev_inst.task_complete_count,
    rev_inst.task_count,
    rev_inst.start_date,
    rev_inst.end_date,
    rev_inst.expected_end_date,
    rev_inst.status,
    rev_inst.status_comment,
    multi_owner.review_owners
  FROM (SELECT
        review_instance.id inst_id,
        defn.id defn_id, 
        defn.unique_review_def_id,
        CASE WHEN staged_defn.name IS NULL THEN defn.name ELSE staged_defn.name END AS name,
        defn.deleted
      FROM review_definition defn
        JOIN review_instance ON defn.id = review_instance.review_definition
        LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn
    JOIN review_instance rev_inst ON defn.inst_id = rev_inst.id AND rev_inst.status NOT IN ('PREVIEW', 'PREVIEW_CANCELED', 'PREVIEW_STARTING')
    LEFT JOIN suser canceled_by ON rev_inst.canceled_by = canceled_by.unique_user_id AND canceled_by.effective = true
    JOIN (SELECT DISTINCT 
        defn_1.name AS review_name,
        ri.id AS rid,
        string_agg(s.display_name, ' | ') AS review_owners
      FROM review_instance ri
        JOIN review_definition defn_1 ON ri.review_definition = defn_1.id
        JOIN reviewer_instance rev_owner ON ri.id = rev_owner.review_instance_id AND rev_owner.type = 'RO'
        LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = rev_owner.id
        LEFT JOIN suser s ON rui.unique_user_id = s.unique_user_id AND s.effective = true
      GROUP BY defn_1.name, ri.id) multi_owner ON rev_inst.id = multi_owner.rid;

ALTER TABLE review_instance_v
  OWNER TO igops;
GRANT ALL ON TABLE review_instance_v TO igops;
GRANT SELECT ON TABLE review_instance_v TO ig_report_role;

ALTER TABLE reviewer_definition_v
  OWNER TO igops;
GRANT ALL ON TABLE reviewer_definition_v TO igops;
GRANT SELECT ON TABLE reviewer_definition_v TO ig_report_role;

ALTER TABLE review_definition_v
  OWNER TO igops;
GRANT ALL ON TABLE review_definition_v TO igops;
GRANT SELECT ON TABLE review_definition_v TO ig_report_role;

ALTER TABLE review_notifications_v
  OWNER TO igops;
GRANT ALL ON TABLE review_notifications_v TO igops;
GRANT SELECT ON TABLE review_notifications_v TO ig_report_role;

ALTER TABLE review_definition_details_v
  OWNER TO igops;
GRANT ALL ON TABLE review_definition_details_v TO igops;
GRANT SELECT ON TABLE review_definition_details_v TO ig_report_role;
